#!/bin/bash

# for f in *.pdf ; do fout=`echo $f | sed 's/.pdf$//g' | sed 's/ /_/g'`; ~/Recherche/Program/Ogmios2/Lingua-Ogmios/bin/pdftoOgmiosXML.pl $f $fout fr > $fout.xml ;done

cat $1 | head -2

cat $1 | grep -v '<?xml version' | grep -v '<documentCollection' | grep -v '</documentCollection'

cat $1 | tail -1

exit;

